import QtQuick 1.0

Item {
    id: container

    // Year item dimensions
    property int yearWidth: (width-2*spacing)*0.4
    property int yearHeight: height
    // Month item dimensions
    property int monthWidth: (width-2*spacing)*0.3
    property int monthHeight: height
    // Day item dimensions
    property int dayWidth: (width-2*spacing)*0.3
    property int dayHeight: height
    // Font properties
    property string fontName: 'Helvetica'
    property int fontSize: 22
    property color fontColor: "#666666"
    // Spacing between items
    property int spacing: 8

    property Component itemBackground: Component {
        BorderImage {
            border { top: 8; bottom: 8; left: 8; right: 8 }
            source: "gfx/button.png"
        }
    }
    property Component itemBackgroundPressed: Component {
        BorderImage {
            border { top: 8; bottom: 8; left: 8; right: 8 }
            source: "gfx/button_pressed.png"
        }
    }

    width: 240
    height:  60

    Component.onCompleted: {
        var date = new Date();
        month.index = date.getMonth();
        day.index = date.getDate()-1;
    }

    Component {
        id: yearDelegate
        Button {
            width: container.yearWidth
            height: container.yearHeight
            text: number
            fontColor: container.fontColor
            fontName: container.fontName
            fontSize: container.fontSize
            bg: itemBackground
            bgPressed: itemBackgroundPressed
            onClicked: { year.index = index; year.toggle() }
        }
    }

    Component {
        id: monthDelegate
        Button {
            width: container.monthWidth
            height: container.monthHeight
            text: number
            fontColor: container.fontColor
            fontName: container.fontName
            fontSize: container.fontSize
            bg: itemBackground
            bgPressed: itemBackgroundPressed
            onClicked: { month.index = index; month.toggle() }
            opacity: (index+1 < months.start) ? 0.5 : 1.0
        }
    }

    Component {
        id: dayDelegate
        Button {
            width: container.dayWidth
            height: container.dayHeight
            text: number
            fontColor: container.fontColor
            fontName: container.fontName
            fontSize: container.fontSize
            bg: itemBackground
            bgPressed: itemBackgroundPressed
            onClicked: { day.index = index; day.toggle() }
            opacity: (index+1 < days.start || index+1 > days.end) ? 0.5 : 1.0
        }
    }
    Row {
        id: reels
        spacing: container.spacing

        Reel {
            id: year
            width: container.yearWidth
            height: container.yearHeight
            model: years
            index: getCurrentYear()

            function getCurrentYear() {
                var today
                today = new Date()
                return today.getYear()
            }

            delegate: yearDelegate
            onIndexChanged: months.update()
            autoClose: false            
        }

        Reel {
            id: month
            width: container.monthWidth
            height: container.monthHeight
            model: months            
            index: getCurrentMonth()

            function getCurrentMonth() {
                var today
                today = new Date()
                return today.getMonth() - 1
            }

            delegate: monthDelegate
            onIndexChanged: { checkIndex(); days.update() }
            autoClose: false
            function checkIndex() {
                if (index+1 < months.start) index = (index+1 < months.start-(index+1)) ? index = 11 : index = months.start-1
            }
        }

        Reel {
            id: day
            width: container.dayWidth
            height: container.dayHeight            
            onIndexChanged: checkIndex()
            model: days
            index: getCurrentDay()

            function getCurrentDay() {
                var today
                today = new Date()
                return today.getDay() - 1
            }

            delegate:  dayDelegate
            autoClose: false
            function checkIndex() {
                if (index+1 < days.start) {
                    index = (31-days.end+index+1 < days.start-(index+1)) ? days.end-1 : days.start-1
                } else if (index + 1 > days.end) {
                    index = (index+1-days.end < 31-(index+1)+days.start) ? days.end-1 : days.start-1
                }
            }
        }
    }


    ListModel{
        id: years        
        Component.onCompleted: {
            months.update();
        }
        ListElement { number: "1900" }
        ListElement { number: "1901" }
        ListElement { number: "1902" }
        ListElement { number: "1903" }
        ListElement { number: "1904" }
        ListElement { number: "1905" }
        ListElement { number: "1906" }
        ListElement { number: "1907" }
        ListElement { number: "1908" }
        ListElement { number: "1909" }
        ListElement { number: "1910" }
        ListElement { number: "1911" }
        ListElement { number: "1912" }
        ListElement { number: "1913" }
        ListElement { number: "1914" }
        ListElement { number: "1915" }
        ListElement { number: "1916" }
        ListElement { number: "1917" }
        ListElement { number: "1918" }
        ListElement { number: "1919" }
        ListElement { number: "1920" }
        ListElement { number: "1921" }
        ListElement { number: "1922" }
        ListElement { number: "1923" }
        ListElement { number: "1924" }
        ListElement { number: "1925" }
        ListElement { number: "1926" }
        ListElement { number: "1927" }
        ListElement { number: "1928" }
        ListElement { number: "1929" }
        ListElement { number: "1930" }
        ListElement { number: "1931" }
        ListElement { number: "1932" }
        ListElement { number: "1933" }
        ListElement { number: "1934" }
        ListElement { number: "1935" }
        ListElement { number: "1936" }
        ListElement { number: "1937" }
        ListElement { number: "1938" }
        ListElement { number: "1939" }
        ListElement { number: "1940" }
        ListElement { number: "1941" }
        ListElement { number: "1942" }
        ListElement { number: "1943" }
        ListElement { number: "1944" }
        ListElement { number: "1945" }
        ListElement { number: "1946" }
        ListElement { number: "1947" }
        ListElement { number: "1948" }
        ListElement { number: "1949" }
        ListElement { number: "1950" }
        ListElement { number: "1951" }
        ListElement { number: "1952" }
        ListElement { number: "1953" }
        ListElement { number: "1954" }
        ListElement { number: "1955" }
        ListElement { number: "1956" }
        ListElement { number: "1957" }
        ListElement { number: "1958" }
        ListElement { number: "1959" }
        ListElement { number: "1960" }
        ListElement { number: "1961" }
        ListElement { number: "1962" }
        ListElement { number: "1963" }
        ListElement { number: "1964" }
        ListElement { number: "1965" }
        ListElement { number: "1966" }
        ListElement { number: "1967" }
        ListElement { number: "1968" }
        ListElement { number: "1969" }
        ListElement { number: "1970" }
        ListElement { number: "1971" }
        ListElement { number: "1972" }
        ListElement { number: "1973" }
        ListElement { number: "1974" }
        ListElement { number: "1975" }
        ListElement { number: "1976" }
        ListElement { number: "1977" }
        ListElement { number: "1978" }
        ListElement { number: "1979" }
        ListElement { number: "1980" }
        ListElement { number: "1981" }
        ListElement { number: "1982" }
        ListElement { number: "1983" }
        ListElement { number: "1984" }
        ListElement { number: "1985" }
        ListElement { number: "1986" }
        ListElement { number: "1987" }
        ListElement { number: "1988" }
        ListElement { number: "1989" }
        ListElement { number: "1990" }
        ListElement { number: "1991" }
        ListElement { number: "1992" }
        ListElement { number: "1993" }
        ListElement { number: "1994" }
        ListElement { number: "1995" }
        ListElement { number: "1996" }
        ListElement { number: "1997" }
        ListElement { number: "1998" }
        ListElement { number: "1999" }
        ListElement { number: "2000" }
        ListElement { number: "2001" }
        ListElement { number: "2002" }
        ListElement { number: "2003" }
        ListElement { number: "2004" }
        ListElement { number: "2005" }
        ListElement { number: "2006" }
        ListElement { number: "2007" }
        ListElement { number: "2008" }
        ListElement { number: "2009" }
        ListElement { number: "2010" }
        ListElement { number: "2011" }
        ListElement { number: "2012" }
        ListElement { number: "2013" }
        ListElement { number: "2014" }
        ListElement { number: "2015" }
        ListElement { number: "2016" }
        ListElement { number: "2017" }
        ListElement { number: "2018" }
        ListElement { number: "2019" }
        ListElement { number: "2020" }
        ListElement { number: "2021" }
        ListElement { number: "2022" }
        ListElement { number: "2023" }
        ListElement { number: "2024" }
        ListElement { number: "2025" }
        ListElement { number: "2026" }
        ListElement { number: "2027" }
        ListElement { number: "2028" }
        ListElement { number: "2029" }
        ListElement { number: "2030" }
        ListElement { number: "2031" }
        ListElement { number: "2032" }
        ListElement { number: "2033" }
        ListElement { number: "2034" }
        ListElement { number: "2035" }
        ListElement { number: "2036" }
        ListElement { number: "2037" }
        ListElement { number: "2038" }
        ListElement { number: "2039" }
        ListElement { number: "2040" }
        ListElement { number: "2041" }
        ListElement { number: "2042" }
        ListElement { number: "2043" }
        ListElement { number: "2044" }
        ListElement { number: "2045" }
        ListElement { number: "2046" }
        ListElement { number: "2047" }
        ListElement { number: "2048" }
        ListElement { number: "2049" }
        ListElement { number: "2050" }
        ListElement { number: "2051" }
        ListElement { number: "2052" }
        ListElement { number: "2053" }
        ListElement { number: "2054" }
        ListElement { number: "2055" }
        ListElement { number: "2056" }
        ListElement { number: "2057" }
        ListElement { number: "2058" }
        ListElement { number: "2059" }
        ListElement { number: "2060" }
        ListElement { number: "2061" }
        ListElement { number: "2062" }
        ListElement { number: "2063" }
        ListElement { number: "2064" }
        ListElement { number: "2065" }
        ListElement { number: "2066" }
        ListElement { number: "2067" }
        ListElement { number: "2068" }
        ListElement { number: "2069" }
        ListElement { number: "2070" }
        ListElement { number: "2071" }
        ListElement { number: "2072" }
        ListElement { number: "2073" }
        ListElement { number: "2074" }
        ListElement { number: "2075" }
        ListElement { number: "2076" }
        ListElement { number: "2077" }
        ListElement { number: "2078" }
        ListElement { number: "2079" }
        ListElement { number: "2080" }
        ListElement { number: "2081" }
        ListElement { number: "2082" }
        ListElement { number: "2083" }
        ListElement { number: "2084" }
        ListElement { number: "2085" }
        ListElement { number: "2086" }
        ListElement { number: "2087" }
        ListElement { number: "2088" }
        ListElement { number: "2089" }
        ListElement { number: "2090" }
        ListElement { number: "2091" }
        ListElement { number: "2092" }
        ListElement { number: "2093" }
        ListElement { number: "2094" }
        ListElement { number: "2095" }
        ListElement { number: "2096" }
        ListElement { number: "2097" }
        ListElement { number: "2098" }
        ListElement { number: "2099" }
    }

    ListModel{
        id: months
        property int start: 1        
        function update() {
            var date = new Date();
            months.start = (year.index === 0) ? date.getMonth()+1 : 1;
            month.checkIndex();
            days.update();
        }

        ListElement { number: "1"; name: "January" }
        ListElement { number: "2"; name: "February" }
        ListElement { number: "3"; name: "March" }
        ListElement { number: "4"; name: "April" }
        ListElement { number: "5"; name: "May" }
        ListElement { number: "6"; name: "June" }
        ListElement { number: "7"; name: "July" }
        ListElement { number: "8"; name: "August" }
        ListElement { number: "9"; name: "September" }
        ListElement { number: "10"; name: "October" }
        ListElement { number: "11"; name: "November" }
        ListElement { number: "12"; name: "December" }
    }

    ListModel {
        id: days
        property int start: 1
        property int end: 31

        function update() {
            var date = new Date();

            var selectedYear = date.getFullYear();
            try { selectedYear = years.get(year.index).number;
            } catch(err) {}

            days.start = 1;
            if (selectedYear == date.getFullYear() && month.index === date.getMonth()) days.start = date.getDate();

            // Determine the amount of days in month
            days.end = 32 - new Date(selectedYear, month.index, 32).getDate();

            if (day.index+1 < days.start) day.index = days.start-1;
            else if (day.index+1 > days.end ) day.index = days.end-1;
        }

        ListElement { number: "1"}
        ListElement { number: "2"}
        ListElement { number: "3"}
        ListElement { number: "4"}
        ListElement { number: "5"}
        ListElement { number: "6"}
        ListElement { number: "7"}
        ListElement { number: "8"}
        ListElement { number: "9"}
        ListElement { number: "10"}
        ListElement { number: "11"}
        ListElement { number: "12"}
        ListElement { number: "13"}
        ListElement { number: "14"}
        ListElement { number: "15"}
        ListElement { number: "16"}
        ListElement { number: "17"}
        ListElement { number: "18"}
        ListElement { number: "19"}
        ListElement { number: "20"}
        ListElement { number: "21"}
        ListElement { number: "22"}
        ListElement { number: "23"}
        ListElement { number: "24"}
        ListElement { number: "25"}
        ListElement { number: "26"}
        ListElement { number: "27"}
        ListElement { number: "28"}
        ListElement { number: "29"}
        ListElement { number: "30"}
        ListElement { number: "31"}
    }

    function getDate() {
        return "" + months.get(month.index).name + " " + days.get(day.index).number

    }
}
